import {
  getBatchProxy,
  getSyntaxProxy,
  runQueriesWithStorageAndHooks
} from "./chunk-4EKVIYPY.js";

// src/syntax/handlers.ts
var queriesHandler = async (queries, optionsFactory = {}) => {
  const options = typeof optionsFactory === "function" ? optionsFactory() : optionsFactory;
  if (!options.token && typeof process !== "undefined") {
    const token2 = typeof process?.env !== "undefined" ? process.env.RONIN_TOKEN : typeof import.meta?.env !== "undefined" ? import.meta.env.RONIN_TOKEN : void 0;
    if (!token2 || token2 === "undefined") {
      const message = "Please specify the `RONIN_TOKEN` environment variable or set the `token` option when invoking RONIN.";
      throw new Error(message);
    }
    options.token = token2;
  }
  const token = options.token;
  if (!token) {
    let message = "When invoking RONIN from an edge runtime, the";
    message += " `token` option must be set.";
    throw new Error(message);
  }
  return runQueriesWithStorageAndHooks(queries, options);
};
var queryHandler = async (query, options) => {
  const results = await queriesHandler([query], options);
  return results[0];
};

// src/syntax/index.ts
var createSyntaxFactory = (options) => ({
  create: getSyntaxProxy(
    "create",
    (query, queryOptions) => queryHandler(query, queryOptions || options)
  ),
  get: getSyntaxProxy(
    "get",
    (query, queryOptions) => queryHandler(query, queryOptions || options)
  ),
  set: getSyntaxProxy(
    "set",
    (query, queryOptions) => queryHandler(query, queryOptions || options)
  ),
  drop: getSyntaxProxy(
    "drop",
    (query, queryOptions) => queryHandler(query, queryOptions || options)
  ),
  count: getSyntaxProxy(
    "count",
    (query, queryOptions) => queryHandler(query, queryOptions || options)
  ),
  batch: (operations, batchQueryOptions) => getBatchProxy(
    operations,
    (queries, queryOptions) => queriesHandler(queries, queryOptions || batchQueryOptions || options)
  )
});

// src/index.ts
var { create, get, set, drop, count, batch } = createSyntaxFactory({});
var src_default = createSyntaxFactory;
export {
  batch,
  count,
  create,
  src_default as default,
  drop,
  get,
  set
};
